﻿Imports VamosALaPlayer_3._0.ViewModel.Instrastructure
Imports System.Collections.ObjectModel
Imports VamosALaPlayer_3._0.Model
Imports System.Windows.Input
Imports System.ComponentModel
Imports System.Windows.Data

Public Class SucheViewModel
    Inherits Infrastructure.ViewModelBase

    Public Sub New()
        InhaltGesamt = New ObservableCollection(Of MP3FileInfo)
        InhaltGesamt.Add(New MP3FileInfo("C:\Test1.mp3", "01", "DMX", "Great Depression", "Trina Moe", "03:49", "2003", False, "Hiphop"))
        InhaltGesamt.Add(New MP3FileInfo("C:\Test2.mp3", "01", "Eminem", "Marshal Mathers LP", "Stan", "02:30", "2000", False, "Hip-Hop"))
        InhaltGesamt.Add(New MP3FileInfo("www.swr3.de/stream", "01", "SWR3", "SWR3", "SWR3", "00:00", DateTime.Now.Year, True, "Rock/Pop"))
        InhaltGesamt.Add(New MP3FileInfo("www.dasding.de/stream", "01", "DasDing", "DasDing", "DasDing", "00:00", DateTime.Now.Year, True, "Dance"))

        GenerateGenreList()

        _mp3FilesView = CollectionViewSource.GetDefaultView(InhaltGesamt.ToArray()) 'View für Musiktitel erstellen (Ob Album oder Interpret angezeigt wird entscheided View durch Binding)
        _streamsView = CollectionViewSource.GetDefaultView(InhaltGesamt.ToArray()) 'View für Streams erstellen
        'Beide Views erhalten erstmal die selbe Datengrundlage, was angezeigt wird entscheidet anschliessend rein der Filter!!

        _mp3FilesView.Filter = AddressOf Mp3FilesView_Filter 'Filtermethode angeben
        _streamsView.Filter = AddressOf StreamsView_Filter 'Filtermethode angeben

        'Sortierungen setzen, dies könnte dann auch über Binding geregelt werden, da könnte es sich der User aussuchen
        _mp3FilesView.SortDescriptions.Add(New SortDescription(NameOf(MP3FileInfo.Musiktitel), ListSortDirection.Ascending))
        _streamsView.SortDescriptions.Add(New SortDescription(NameOf(MP3FileInfo.Interpret), ListSortDirection.Ascending))
    End Sub

    Private _TextFilter As String = ""
    Public Property TextFilter() As String
        Get
            Return _TextFilter
        End Get
        Set(ByVal value As String)
            _TextFilter = value : RaisePropertyChanged() : RefreshViews()
        End Set
    End Property

    Private Property InhaltGesamt() As ObservableCollection(Of MP3FileInfo) 'Property ist Private und muss garnicht nach aussen gereicht werden da von der View nicht benötigt

    Private _mp3FilesView As ICollectionView 'View für Musiktitel
    Public ReadOnly Property Mp3FilesView As ICollectionView
        Get
            Return _mp3FilesView
        End Get
    End Property

    Private _streamsView As ICollectionView 'View für Streams
    Public ReadOnly Property StreamsView As ICollectionView
        Get
            Return _streamsView
        End Get
    End Property

    Private Function StreamsView_Filter(obj As Object) As Boolean 'Die Methode filtert alle Musiktitel und alles was weder = dem TextFilter ist noch ein abgewähltes Genre ist
        Dim mp3Object = DirectCast(obj, MP3FileInfo)
        Dim allowed = mp3Object.Stream
        If Not String.IsNullOrEmpty(TextFilter) Then
            allowed = mp3Object.Interpret.ToLower().Contains(TextFilter.ToLower())
            If Not allowed Then Return allowed
        End If
        For Each selGenre In AnzuzeigendeGenres.Where(Function(g) Not g.IsSelected)
            If mp3Object.Genre = selGenre.Genre Then Return False
        Next

        Return allowed
    End Function

    Private Function Mp3FilesView_Filter(obj As Object) As Boolean 'Die Methode filtert alle Streams und alles was weder = dem TextFilter ist noch ein abgewähltes Genre ist
        Dim mp3Object = DirectCast(obj, MP3FileInfo)
        Dim allowed = Not mp3Object.Stream
        If Not allowed Then Return allowed
        If Not String.IsNullOrEmpty(TextFilter) Then
            allowed = mp3Object.Interpret.ToLower().Contains(TextFilter.ToLower()) OrElse mp3Object.Album.ToLower().Contains(TextFilter.ToLower())
            If Not allowed Then Return allowed
        End If
        For Each selGenre In AnzuzeigendeGenres.Where(Function(g) Not g.IsSelected)
            If mp3Object.Genre = selGenre.Genre Then Return False
        Next

        Return allowed
    End Function

    Private Sub GenerateGenreList() 'Generiert die Genre-Liste mit dem zusätzlichem Property IsSelected um dieses später auswertuen zu können
        Dim aviableGenres = InhaltGesamt.Select(Function(s) s.Genre).Distinct().ToList() 'Erstmal die alle Genres holen aus den MP3 holen (Als Modelobjekte) Distinct() filtert alle Doppelten einträge
        Dim returnList = New List(Of GenreViewModel)
        aviableGenres.ForEach(Sub(x) returnList.Add(New GenreViewModel(New Genres(x)))) 'Nun aus den Modelobjekten ViewModel Objekte machen
        returnList.ForEach(Sub(x) AddHandler x.IsSelectedChanged, AddressOf SelectedGenresChanged) 'Nun für jedes Objekt einen Handler auf das Event in der GenreViewModel-Klasse setzen 
        _anzuzeigendeGenres = New ObservableCollection(Of GenreViewModel)(returnList)
        RaisePropertyChanged(NameOf(AnzuzeigendeGenres))
    End Sub

    Private _anzuzeigendeGenres As ObservableCollection(Of GenreViewModel)
    Public ReadOnly Property AnzuzeigendeGenres As ObservableCollection(Of GenreViewModel)
        Get
            Return _anzuzeigendeGenres
        End Get
    End Property

    Private Sub SelectedGenresChanged()
        RefreshViews()
    End Sub


    Private Sub RefreshViews() 'Aktualisiert die Views
        Mp3FilesView.Refresh() : StreamsView.Refresh()
    End Sub
End Class
